PROGRAM bmain
!
!*******************************************************************************
! Solve FH incomplete bond economy using value function iteration
!
! Incomplete market with state-uncontigent bond, no adjustment cost
! 
! Utility function: u(c)=(c^(1-sigma)-1)/(1-sigma)
! No aggregate uncertainty
! productivity shock
! Related code: val.m
! Problem: V(A,b,k) = max (1-beta)(u(c))+beta* sum (p(A'|A)*v(A',b',k')
!                     s.t. c = A * k^alpha + (1-delta) * k + R * b - b' - k'
!                          b' >= B(a,k')
! where, B(a,k') is the endogenous borrowing constraints from limited enforcement.
!
! Author: Bai and Zhang
! March 20, 2009
!******************************************************************************

USE bvar

IMPLICIT NONE
INCLUDE 'mpif.h'

real*8                               :: es
integer                              :: i,j,m
integer, dimension(la,lb,lk)         :: setib,setik
real*8, dimension(la,lb,lk)          :: v0,setkp,setbp, setc

integer, dimension(la*lb*lk)         :: setibp,setikp
real*8, dimension(la*lb*lk)          :: den


real*8                               :: sav, gdp,deb,dratio


! Input to the model
open(unit=7,  file =trim(dirin)//trim(sfile))

open(unit=3,  file=trim(dirout)//'para.dat')
open(unit=13, file=trim(dirout)//'kgrid.dat')
open(unit=15, file =trim(dirout)//'bgrid.dat')
open(unit=17, file =trim(dirout)//'autk.dat')

! Output to the model
open(unit=4,  file =trim(dirout)//'shock.dat')
open(unit=5,  file =trim(dirout)//'vbondout.txt')
open(unit=16, file =trim(dirout)//'bondV.dat')

open(unit=8,  file =trim(dirout)//'bondKp.dat')
open(unit=9,  file =trim(dirout)//'bondBp.dat')
open(unit=10, file =trim(dirout)//'bondSai.dat')
open(unit=12, file =trim(dirout)//'ibondKp.dat')
open(unit=14, file =trim(dirout)//'ibondBp.dat')
open(unit=20, file =trim(dirout)//'bondC.dat')

open(unit=30, file =trim(dirout)//'autU.dat')
open(unit=32, file =trim(dirout)//'autkp.dat')
open(unit=40, file =trim(dirout)//'bbar.dat')
open(unit=42, file =trim(dirout)//'ibbar.dat')
open(unit=44, file =trim(dirout)//'test.dat')
open(unit=46, file =trim(dirout)//'psk.dat')
open(unit=50, file =trim(dirout)//'tbbar.dat')
open(unit=100, file =trim(dirout)//'tmp.dat')

!*******************************************************************

! Initial MPI process
 call MPI_INIT(ierr)

! Get my rank
 call MPI_COMM_RANK(MPI_COMM_WORLD, myrank, ierr)
 call MPI_COMM_SIZE(MPI_COMM_WORLD, mysize, ierr)


talpha   = 1.0/(1-alpha)

!*******************************************************************
! Allocat works to different processors
do i = 1, np
	istart(i) = (i-1)*mk+1
end do

do i = 1,np
	if (myrank == i-1) then
		inda  = istart(i)
		!print *, 'I am processor ', myrank, ' and I will compute from ', inda
	end if
end do

!*******************************************************************
! Begin solving the problem
! First iterate beta to clear markets at R=1.04
! Second iterate over value function
! Last iterate over distribution!

call breadin(v0,setikp,setibp)

if (ibetaR == 2) then
	call biterR(v0,setib,setik,setkp,setbp,setc,setikp,setibp,es,sav,gdp,deb,dratio,den)
elseif (ibetaR==1) then
	call biterBeta(v0,setib,setik,setkp,setbp,setc,setikp,setibp,es,sav,gdp,deb,dratio,den)
end if

if (myrank==1) then
	call bwrite(v0,setib,setik,setkp,setbp,setc,setikp,setibp,es,sav,gdp,deb,dratio,den)
	print *, 'Done Everything!!'
end if
 
 
call MPI_FINALIZE(ierr)
END PROGRAM bmain








































