subroutine autarkyv

!**********************************************************************************
! Solve neoclassical growth model in a closed economy 
! 
! Utility function: u(c)=(c^(1-sigma)-1)/(1-sigma)
! No aggregate uncertainty
! Productivity shock
!
! Problem: 
! V(A,K)=max (1-beta)(u(c))+beta* sum (p(A'|A)*V(A', K')
!          s.t. c = A*k^alpha + (1-delta)*k - k'
!
! Solve the problem using value function iterations
!
! Author: Bai and Zhang
! February 18, 2009
!********************************************************************************

Use bondenforcevar

IMPLICIT NONE
INCLUDE 'mpif.h'

real*8                          :: gridk, dist, ttv, ttv1
real*8                          :: vv(la*autlk), vv1(la*autlk), vv0(la*autlk), autEv(autlk)
integer                         :: i, j, m, ii, jj, rr

real*8, dimension(la,autlk)     :: uaut0, uaut
real*8, dimension(autlk,autlk)  :: autkk, autkkp, autinv, auttvv, tmpv, c, u
real*8, dimension(la*autlk)     :: uu1
real*8, dimension(la*lk)        :: uu2


! *************************************************************
! Initital the capital grid
gridk    = (autkmax-autkmin)/(autlk*(autlk-1)/2.0)
autk(1)  = autkmin
do i=2,autlk
	autk(i) = autk(i-1) + (i-1)*gridk
end do

! Compute investment
autkk   = spread(autk,1,autlk)          ! k
autkkp  = transpose(autkk)              ! kkp
autinv  = autkkp - (1-delta)*autkk + gama*(autkkp-autkk)**2.0/autkk

! ****************************************************************
! Start to solve the autarky utility
if (readu==1) then
	rewind(30)
	do i = 1, la
		do j = 1, autlk
			read(30,*) uaut0(i,j)
		end do
	end do
	rewind(30)
	uaut = uaut0
else
	!*************************************************************
	! Initial value function
	uaut0     = pen
	vv0       = pen
	!*************************************************************
	! Begin solving the autarky utility
	if (myrank == 0) then
		print *,'Start solving autarky economy using value function iteration:'
	end if
	
	dist  = 2.0
	do
		if (dist <= critv) exit

		do i=1,la
			c  = (1.0-lamda)*A(i)*autkk**alpha-autinv
			where (c > 0.0) 
				u  = (c**(1.0-sigma)-1.0)/(1.0-sigma)
			elsewhere
				u  = pen
			endwhere
			autEv          = matmul(p(i,:),uaut0)
			auttvv         = spread(autEv,2,autlk)
        	tmpv           = u + rbeta*auttvv
			uaut(i,:)      = maxval(tmpv,1)
		end do                          ! end iterate over na
	
		! Exit criterion if policy rules converges
		ttv         = sqrt(sum((uaut-uaut0)**2))
		ttv1        = 1.0 + sqrt(sum(uaut0**2))
		dist        = ttv / ttv1
		vv0         = vv1		
		uaut0       = uaut

!		if (myrank == 0) then
			!print *,dist
!		end if
	end do ! End of value function iteration
end if
	
uu   = uaut


END subroutine autarkyv





















