Program SimuMain

!*******************************************************
! Simulate and test for bond-enforcement model
!
! Input:
!       Productivity shock process A and P
!       Invariant distribution over states (A,K,B): miu (in CRS format)
!       Decision rule: gb, gk
!       State grids  : k,  b
!
!       miu(la*lk*lb):        stationary distribution
!       gb(la,lb,lk) :        bond holding decision bp
!       gk(la,lb,lk) :        capital holding decision kp
!
! Output:
!       solve the complete market
!       simulate the model
!       generate the results of simulation
!       Run FH regression
!
!       fhcoef       :        regression coefficient of FH puzzle for each simulation
!       statistics for fhcoef
!
!   N:                        number of simulations
!   T:                        number of years to be simulated
!   NC:                       number of countries to be simulated
!
! March 12, 2009 by Bai and Zhang
!*******************************************************


USE SimuVar
USE SimuRs
implicit none

! Simulation results
integer, dimension(T, nc)    :: isimua
integer, dimension(1, nc)    :: isimuk0, isimub0
real*8, dimension(T, nc)     :: simua
real*8, dimension(1, nc)     :: simuk0, simub0
real*8, dimension(T, nc)     :: simuk, simuc, simuy, simui, simus, simub, simukk, simubb,simul

real*8,dimension(T,nc,N):: rk
real*8,dimension(T-1,nc,N):: mrs
real*8:: rr(nc,N)
! Statistics of simulation

real*8, dimension(nc)        :: fhsy, fhiy
real*8,dimension(N)          :: setb, stdb, relvol,tby                     
real*8                       :: b0, b1, se, mrelvol,tmean
real*8, dimension(12)        :: stat1
character(8)                 :: tdate
character(10)                :: ttime


! Working arrays
integer                     :: i, j, m, ii, jj, s, cut, tn, iwt,nvol,ilog,mm
integer                     :: ienf
real*8                      :: temp, temp1, kp, bp, tmp, tmp1,lp,mky(N),stdky(N)
real*8, dimension(2)        :: valk,valb,afhsy(N),afhiy(N),sfhsy(N),sfhiy(N),vala
integer, dimension(2)       :: indk,indb,inda
real*8                      :: volgdp(N,nc), volcon(N,nc), volinv(N,nc),mvolgdp,rvolinv,rvolcon,kyratio(N,nc)
real*8                      :: corr(N), tcorr(2,2), xx(nc,2),mtime(N,ntvar),stime(N,ntvar)
integer                     :: nvar
real*8                      :: amtime(ntvar),smtime(ntvar),mstime(ntvar),sstime(ntvar),cay(N),tay(N)


real*8                      :: gbc(N,3,2),gs1(NC), gs2(NC), gs3(NC),gi1(NC), gi2(NC), gi3(NC)
real*8                      :: agbc1(3), agbc2(3), sgbc1(3), sgbc2(3)
integer                     :: ig1,ig2,ig3

! Risksharing statistics
real*8                       :: poolb(N,3),pdifb(N,2),mcgc(N),mmcgc,stp,yy
real*8                       :: csb(N),std_csb(N),scsb(N,T-1),sstd_csb(N,T-1)
real*8                       :: mpoolb,stdpoolb,mcsb,mstdcsb,mpdfib,stdpdifb,gdp,deb
real*8                       :: twelc(T,NC), ttwelc(NC),cc(NC),welc(N),wwc,mmc

! Consumption growth rate
real*8,dimension(T-1,nc)     :: grc,gry
real*8,dimension(T)          :: mc,my
real*8,dimension(T-1)        :: grmc,grmy     
real*8,dimension(nc)         :: mgrc,mgry
real*8:: pby(N),nby(N),nnb,ppb

real*8:: trend(T), diff(T)

! correlation and dispersion
real*8:: sgdp(NC),csys(N),mcsys,stdcsys,csyi(N),mcsyi, stdcsyi
real*8:: dsav(N),dinv(N),dgdp(N) ! dispersions
real*8:: per(3,2),tgdp(NC),tfhiy(NC),tfhsy(NC),mgdp(N),stay(N),scay(N)
real*8:: mcay(NC),mtay(NC)
integer:: l5,u5

external simubase 

! Input files
open(unit=1,   file=trim(dirin)//'shock.dat')     
open(unit=3,   file=trim(dirin)//'bondKp.dat')     
open(unit=5,   file=trim(dirin)//'bondBp.dat')     
open(unit=7,   file=trim(dirin)//'ibondKp.dat')
open(unit=9,   file=trim(dirin)//'ibondBp.dat')
open(unit=11,  file=trim(dirin)//'bondSai.dat')     
open(unit=13,  file=trim(dirin)//'kgrid.dat')   
open(unit=15,  file=trim(dirin)//'bgrid.dat')
open(unit=17,  file=trim(dirin)//'para.dat')   
	
read(17,*) beta1
read(17,*) sigma
read(17,*) alpha
read(17,*) delta
read(17,*) R
read(17,*) la
read(17,*) lk
read(17,*) lb
read(17,*) gama

tn       = la*lb*lk
talpha   = 1.0/(1-alpha)

allocate(A(la),Ea(la),K(lk),b(lb),miu(tn),cummiu(tn),p(la,la),cumpai(la,la),gk(lb,lk,la),gb(lb,lk,la),igk(lb,lk,la),igb(lb,lk,la))

!****************************************************
! Read in state spaces
! A grid
do i=1,la
	read(1,*) a(i)
end do

do i=1,la
	do j=1,la
		read(1,*) p(i,j)
	end do
end do

do i=1, la
	Ea(i)=sum(p(i,:)*a)
end do

! K grid
do i=1,lk
	read(13,*) K(i)
end do

! B grid
do i=1,lb
	read(15,*) b(i)
end do

!*******************************************************************
! Load in bond market decision rules: capital and bond
rewind(3)
rewind(5)

do i=1,la
	do j=1,lb
		do m = 1,lk
			read(3,*) gk(j,m,i)
			read(5,*) gb(j,m,i)
		end do
	end do
end do
rewind(3)
rewind(5)

! Load in invariant distribution
rewind(11)
do i = 1, tn
	read(11, *) miu(i)
end do
rewind(11)

!******************************************************
! Start simulating the model
! Cumulative sum of transition matrix

cumpai(:,1)  = p(:,1)
do i = 2,la
	cumpai(:,i)   = cumpai(:,i-1)+p(:,i)
end do

cummiu(1)=miu(1)
do i=2,la*lk*lb
	cummiu(i)=cummiu(i-1)+miu(i)
end do

!******************************************************
do s=1,N
	call Simu(simua, isimua, simuk0, isimuk0, simub0, isimub0)
	
	do i=1,nc
		do j=1,T
			if (j==1) then 
				simukk(j,i)     = simuk0(1,i)
				simubb(j,i)     = simub0(1,i)
			else
				simukk(j,i)     = simuk(j-1,i)
				simubb(j,i)     = simub(j-1,i)
			end if
			call simubase(simukk(j,i),simubb(j,i),simua(j,i), valk, valb, indk, indb,vala,inda)
			
			kp                  = 0.0
			bp                  = 0.0
			
			do ii = 1, 2
				do jj = 1, 2
					do mm=1,2
						
						kp          = kp + gk(indb(ii),indk(jj),inda(mm)) * valk(jj)*valb(ii)*vala(mm)
						bp          = bp + gb(indb(ii),indk(jj),inda(mm)) * valk(jj)*valb(ii)*vala(mm)
					end do
				end do
			end do
			
			simuk(j,i)          = kp
			simub(j,i)          = bp
				
			simuy(j,i)     = simua(j,i) * simukk(j,i)**alpha
			simui(j,i)      = simuk(j,i) - (1.0 - delta) * simukk(j,i)+gama*(simuk(j,i)/simukk(j,i)-1+delta)**2.0&
*simukk(j,i)/2.0
 
			simuc(j,i)     = simuy(j,i) + R*simubb(j,i) - simui(j,i)-simub(j,i)
			simus(j,i)      = simuy(j,i) - simuc(j,i)
			

		end do                         ! end of simulation for T
		! capital output ratio	
		kyratio(s,i) = sum(simuk(:,i)/simuy(:,i))/real(T)
		! average capital flows across countries
		mcay(i)= sum(abs(simubb(:,i)-simub(:,i))/simuy(:,i))/real(T)
		mtay(i)=sum(abs(simubb(:,i)/simuy(:,i)))/real(T)

	end do                     ! end of simulation for nc 
	
	! each round capital flow across countries
	call meanstd(NC, mcay, cay(s),scay(s))
	call meanstd(NC, mtay, tay(s),stay(s))
	
	mky(s)    = sum(kyratio(s,:))/real(nc)
	stdky(s)  = sqrt(sum((kyratio(s,:)-mky(s))**2.0)/real(nc-1))
	
	
	! Cross section regression for each year
	fhiy  = sum(simui/simuy,1)/real(T)
	fhsy  = sum(simus/simuy,1)/real(T)
	afhiy(s)=sum(fhiy)/real(nc)
	afhsy(s)=sum(fhsy)/real(nc)
	sfhiy(s)=sqrt(sum((fhiy-afhiy(s))**2)/real(nc-1))
	sfhsy(s)=sqrt(sum((fhsy-afhsy(s))**2)/real(nc-1))
	
	call drline(nc,fhsy, fhiy, b0, b1, stat1)
	setb(s)  = b1
	stdb(s)  = stat1(7)
	corr(s) = stat1(5)
		
	! Time-series statistics
	call  timeStat(simuc,simuy,simui,simuk,mtime(s,:),stime(s,:))
	
	!********************
	! Risk Sharing
	! Annual GDP and consumption growth rate for each country
	if (T>=2) then
		grc=log(simuc(2:T,:))-log(simuc(1:T-1,:))
	        gry=log(simuy(2:T,:))-log(simuy(1:T-1,:))
	        
	        ! Average consumption and GDP across countries at each year
	        mc = sum(simuc,2)
		my = sum(simuy,2)
		grmc=log(mc(2:T))-log(mc(1:T-1))
		grmy=log(my(2:T))-log(my(1:T-1))
	
		call PanelRegression(grc,gry,grmc,grmy,poolb(s,:),pdifb(s,:),mcgc(s))
	end if
	
	!********************
	! correlation between output and investment and savings
	! and  dispersion
	
	sgdp=100*((simuy(T,:)/simuy(1,:))**(1/real(T-1))-1)
	call drline(nc,sgdp,fhiy, b0, b1, stat1)
	csyi(s) = stat1(5)
	dinv(s) = sqrt(stat1(4))
	dgdp(s) = sqrt(stat1(3))
	mgdp(s) = stat1(1)
	
	call drline(nc,sgdp,fhsy, b0, b1, stat1)
	csys(s) = stat1(5)
	dsav(s) = sqrt(stat1(4))
	call  dsvrgn(NC,fhiy,tfhiy)
	call  dsvrgn(NC,fhsy,tfhsy)
	call  dsvrgn(NC,sgdp,tgdp)
	
	l5=floor(NC*0.10)
	u5=floor(NC*0.90)
	
	per(1,1)=sum(tfhiy(1:l5))/real(l5)
	
	per(1,2)=sum(tfhiy(u5:NC))/real(NC-u5+1)
	per(2,1)=sum(tfhsy(1:l5))/real(l5)
	per(2,2)=sum(tfhsy(u5:NC))/real(NC-u5+1)
	per(3,1)=sum(tgdp(1:l5))/real(l5)
	per(3,2)=sum(tgdp(u5:NC))/real(NC-u5+1)

	
end do                                 ! end of N

call meanstd(N,pdifb(:,2),mpdfib,stdpdifb)
call meanstd(N,mcgc,mmcgc,stp)
call meanstd(N,csys,mcsys,stdcsys)
call meanstd(N,csyi,mcsyi,stdcsyi)

!**********************************************
write (*,12)
write (*, *)  '******** Bond-Enforcement Model Simulation Report ********'
call date_and_time(tdate,ttime)
write(*, *) '   Date: ', tdate, '   Time: ', ttime
write (*, 12) 
write (*, 20) '  # of simulation:    ', N
write (*, 20) '  # of periods:       ', T
write (*, 20) '  # of countries:     ', NC

write (*,12)

write(*, 16)  '  FH coef:                          ', sum(setb)/real(N),sum(stdb)/real(N)
write(*, 16)  '  CA/Y:                              ',  sum(cay)/real(N), sum(scay)/real(N)
write(*, 16)  '  TA/Y:                               ', sum(tay)/real(N), sum(stay)/real(N)
write(*, 16)  '  Risk sharing                   ', mpdfib,stdpdifb
write(*, 12)
write(*, *) '-------------------------------' 
write(*,*)  ' Cross-country mean:           '
write(*,14)  '   Investment rate:            ', sum(afhiy)/real(N)
write(*,14)  '   Saving rate:                ', sum(afhsy)/real(N)
write(*,14)  '   GDP growth rate:            ', sum(mgdp)/real(N)
write(*,*)
write(*, *) ' Cross-country correlation:    '
write(*, 16) '   GDP growth and investment rate: ', mcsyi,stdcsyi
write(*, 16) '   GDP growth and savings rate:    ', mcsys, stdcsys
write(*,*)
write(*, *) ' Cross-country dispersion:   '
write(*, 14) '   Std of average investment rate: ', sum(dinv)/real(N)
write(*, 14) '   Std of average savings rate:    ', sum(dsav)/real(N)
write(*, 14) '   Std of GDP growth rate:           ', sum(dgdp)/real(N)
write(*,*)
write(*,*)  '        Investment      Saving       GDP growth'
write(*,902)  'Min 10  ', per(1,1),per(2,1),per(3,1)
write(*,902)  'Max 10  ', per(1,2),per(2,2),per(3,2)


do i=1,ntvar
	call meanstd(N,mtime(:,i),amtime(i),smtime(i))
	call meanstd(N,stime(:,i),mstime(i),sstime(i))
end do

write(*,*) '---- Times Series Statistics ------'
write(*,900) 'vol(y)                      ',amtime(2)*100,sstime(2)*100
write(*,904) 'vol(ne/y)                ',amtime(5)*100,sstime(5)*100
write(*,900) 'vol(c)/vol(y)            ',amtime(1),sstime(1)
write(*,904) 'vol(i)/vol(y)             ',amtime(10),  sstime(10)
write(*,900) 'corr(y,c)                 ',amtime(3),sstime(3)
write(*,900) 'corr(y,i)                  ',amtime(9),sstime(9)
write(*,900) 'corr(y,ne)              ',amtime(4),sstime(4)
write(*,900) 'corr(s,i)                 ',amtime(11), sstime(11)
write(*,900) 'corr(yi,yj)               ',amtime(8),sstime(8)
write(*,900) 'corr(ci,cj)              ',amtime(7),sstime(7)
	
12 format(/)
14 format(a40,F7.2)
16 format(a40,F7.2,' (',F4.2,')')
20 format(a40,I2)
102  format(A40, f7.2,f7.2)
900 format(A40,f7.2,'(',F4.2,')')
902 format(A10,f7.2,'      ',f7.2,'       ',f7.2)
904 format(A40,f7.2,'(',F5.2,')')

write(*, *) '***********************************************'

deallocate(A,Ea,K,b,miu,cummiu,p,cumpai,gk,gb,igk,igb)

END program SimuMain
