!=======================================================================
!
subroutine hpfilter(y,trend,diff,n,w)
!
!     SR: hpfilt
!     Kalman smoothing routine for HP filter written by E Pres!ott.
!     y=data series, d=deviations from trend, t=trend, n=no obs,
!     s=smoothing parameter (eg, 1600 for std HP).
!     If iopt=1 and n and s both the same as for last !all, subroutine
!     uses values of v from the last !all. If this option is used, v must
!     not be use for other purposes between last and !urrent !all.
!
implicit none
!


!function [s]=hpfilter(y,w)
!  function [s]=hpfilter(y,w)
!  Hondrick Prescott filter where:
!  w - smoothing parameter; w=1600 for quarterly data
!  y - the original series that has to be smoothed
!  s - the filtered series
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
integer, intent(in):: n
real*8, intent(in)::y(n),w
real*8, intent(out)::trend(n),diff(n)

real*8:: a,b,c,d(3),md(n,3)
real*8,dimension(n,n):: md1,md2,md3,md4,md5,m
integer::i,j


a=6*w+1 
b=-4*w 
c=w 
d(1)=c
d(2)=b
d(3)=a

md=spread(d,1,n)
md1 = 0.0
md2 = 0.0
md3 = 0.0 
md4 = 0.0
md5 =0.0
do i=1,n
	md1(i,i)=md(i,3)
	if (i<n) then
		md2(i,i+1) = md(i,2)
		md3(i+1,i) = md(i,2)
	end if
	if (i<n-1) then
		md4(i,i+2)=md(i,1)
		md5(i+2,i)=md(i,1)
	end if
end do
m=md1+md2+md3 
m=m+md4+md5


m(1,1)=1.0+w
m(1,2)=-2.0*w 
m(2,1)=-2.0*w
m(2,2)=5.0*w+1.0 
m(n-1,n-1)=5.0*w+1
m(n-1,n)=-2.0*w 
m(n,n-1)=-2.0*w
m(n,n)=1.0+w

call DLSLRG (n,m,n, y, 1,trend)
diff = y-trend 
end subroutine hpfilter

