subroutine dist

!******************************************************************
! Solve invariant distribution and excess demand for the complete markets
!       with capital adjustment cost
!
! Dec 22 2004 Jing
!******************************************************************

USE CostVar

IMPLICIT NONE

real, parameter                         :: crit2 = 1.0e-6

integer                                 :: i, j, m, ii, jj, mm, tn
real                                    :: tt, tt1, disp2, bp, temp, kp
real, dimension(la*lk)                  :: miu1

real, dimension(2)                      :: valk
integer, dimension(2)                   :: indk

character(8)                            :: running

! Variables for Sparskits
integer                                 :: indsai,indcol
integer, dimension((la*lk+1)*20)        :: jsai1
real, dimension((la*lk+1)*20)           :: sai1
integer, dimension(la*lk+1)             :: isai,idsai
real, allocatable, dimension(:)         :: sai,dsai
integer, allocatable, dimension(:)      :: jsai,jdsai

external distbase

tn    = la*lk

! Initialize the density
miu  = 0.0
do i = 1, la
	do j = lk/2+1,lk
		miu((i-1)*lk+j)   = 1.0/real(tn/2)
	end do
end do

		
disp2=2.0

print *, 'solving distribution'
	
! Store sai matrix in CSR format
indsai=1		! Counting the total number of nonzeros in sai
indcol=1		! Counting the number of columns

do i=1,la
	do m=1,lk	
		isai(indcol)     = indsai
		kp               = gk(i,m)		! Find kp

		call distbase(kp,valk,indk)

		do ii=1,la			         ! Iteration over a'
			if (p(i,ii)/=0.0) then
				do jj=1,2            ! Iterate over k'
					temp = p(i,ii) * valk(jj)
					if (temp/=0.0) then
						sai1(indsai)    = temp
						jsai1(indsai)   = (ii-1)*lk + indk(jj)
						indsai          = indsai+1
					end if
				end do          ! End of iteration over k'
			end if 
		end do ! End of iteration over a'
		indcol=indcol+1
	end do ! End of iteration over k
end do ! End of iteration over a

allocate(sai(indsai-1),jsai(indsai-1))
allocate(dsai(indsai-1),jdsai(indsai-1))

sai=sai1(1:indsai-1)
jsai=jsai1(1:indsai-1)

call csrcsc(tn,1,1,sai,jsai,isai,dsai,jdsai,idsai) !Transform CSC to CSR format
	
do 
	if (disp2<crit2) exit
	call amux(tn,miu,miu1,dsai,jdsai,idsai)
	tt1   = sqrt(sum((miu-miu1)**2))
	tt    = sqrt(sum((miu)**2))
	disp2 = tt1/tt
	!print *, disp2
	miu   = miu1
end do

rewind(16)
do i=1,tn
	write(16,*) miu(i)
end do
rewind(16)

print *, 'sum of distribution:', sum(miu)
print *, 'maximum density in distribution: ', maxval(miu)

deallocate(sai,jsai,dsai,jdsai)

end subroutine dist

