Subroutine solvegk(xguess,fval,nx)

!**********************************************************************
! June 17, 2004 Jing
!
! Update policy function using derivative vk and vb of value function
!
! Input : xguess (c and bp)
!         nx     (dimension of unknowns)
! Output: fval 
!
! f1    =   lhs - beta * sum(p(i,ii)*vk(a,k,b))
! f2    =   lhs - beta * sum(p(i,ii)*vb(a,k,b))
!
! lhs   = c**(-sigma)
!
! Need to take care of the case where c < 0.0 (technically c < cmin)
! Consider using taylor rule to extend the function c**(-sigma) as
!   lhs = cmin**(-sigma) + sigma * cmin ** (-sigma-1) * (cmin - c)
!
! Basep taking care of the case where k<0 or bp<bmin
!
! Subroutine used:
!	basep(kp,bp,valk,valb,indk,indb):
!       input  : kp bp
!       output : indk and indb (index of k and b interval which (kp, bp) falls into)
!                valk and valb (weight of each corner points)
!
!***********************************************************************

USE CostVar
integer, intent(in)            :: nx
real, intent(in)               :: xguess
real, intent(out)              :: fval

integer                        :: mnn, ii
real                           :: kp,temp1,temp2, temp

real, dimension(2)             :: valk
integer, dimension(2)          :: indk

external base

kp     = xguess

if (kp <= 0.0) then
	fval  = 1.0e6

else
	! Linear interpolation
	call base(kp,valk,indk)

	! Compute residual function
	temp1  = 0.0
	do ii = 1, la        !A'
		temp2     = 0.0	
		do mmm=1,2       !k
			temp2 = temp2 + gk0(ii,indk(mmm))*valk(mmm)
		end do
		temp1  = temp1  + p(ia,ii) * temp2**2.0
	end do

	temp   = beta1*alpha*Ea(ia)*kp**(alpha+1.0) - beta1*gama*temp1/2.0

	fval   = (1.0-gama-beta1*(1.0-delta-gama/2.0))*kp**2.0 + gama*kp**3.0/k(ik) - temp
end if
if (iwrite == 1) then
	write(30,*) xguess, fval
end if
End subroutine solvegk
