Subroutine bisec(xguess, step, critf, itermax, x)
					

!**********************************************************************
! June 27, 2004 Jing
!
! Update policy function using derivative vk and vb of value function
!
! Enforcement constraints are binding for 5 top a'
!
! Input : xguess [gw(a')   for a' such that p(a'|a)>0.0 and enf constraint is not binding
!                [gama(a') for a' such that enf constraints are binding
!                [kp       
!
!         nx     (dimension of unknowns)
! Output: fval   (dimension of nx)
!
! f(i)      =   lhs(i) + rhs(i)
! 
! f(end)    =   (1-1/R)(1-1/R*sum(p(a'|a)*(1-delta+alpha*a'*kp**(alpha-1)))) 
!                + sum( gamma(a') * duaut(a',kp)/dkp )
! 
! lhs(i)    =   -(1-1/R)*beta*p(a'|a)* c'(h)/(1-beta)  
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a'))               if icon(i) == 0         
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a')) - gamma(a')     if icon(i) == 1         
!
! Need to take care of the case where h>hmax (technically need (h(1-sigma)+1)>0.0)
! Consider using taylor rule to extend the function c'(h) as
!   c'(h) = c'(hmax) + c''(hmax)* (h - hmax)
!
! Base taking care of the case where wp<wmin or wp<wmax
!
! Subroutine used:
!	base(wp, valw, indw):
!       input  : wp
!       output : indw (index of W interval which wp falls into)
!                valw (weight of each corner points)
!
!***********************************************************************
USE penfvar
implicit none

real, intent(in)               :: xguess, critf, step
integer, intent(in)            :: itermax
real, intent(out)              :: x

integer, parameter             :: iterstep = 10
real                           :: xmin, xmax, kp, fval, fval1

integer                        :: iters

external compf

kp               = xguess

if (kp< kmin) then
	print *, 'Capital lower bound is violated!'
	stop
end if

call compf(kp, fval)

if (abs(fval) < critf) then
	x = kp
else
	if (fval < 0.0) then
		xmin = kp
		fval1 = -2.0
		iters = 1
		do
			if (fval1 > 0.0 .or. iters > itermax) exit
			kp  = kp * 1.01
			call compf(kp, fval1) 
			iters = iters + 1
		end do
		
		if (fval1 >= 0.0 ) then
			xmax = kp
		else
			print *, 'increase itermax'
			stop
		end if

	else
		xmax = kp
		! Find xmin then
		fval1 = 2.0
		iters = 1
		do
			if (fval1 < 0.0 .or. iters > itermax) exit
			kp  = kp * step
			call compf(kp, fval1) 
			iters = iters + 1
		end do
		
		if (fval1 < 0.0 ) then
			xmin = kp
		else
			print *, 'increase itermax'
			stop
		end if
	end if
	
	fval  = 2.0
	iters = 1
	do
		if (abs(fval) < critf) then
			x = kp
			exit
		end if
		if ( iters > itermax ) then
			print *, 'increase the maximum iteration number'
			stop
		end if
		kp = (xmin + xmax)/2.0
		call compf(kp,fval)
		if (fval > 0.0) then
			xmax = kp
		else
			xmin = kp
		end if
		iters = iters + 1
	end do
	
end if

End subroutine bisec