Subroutine bisec0(xguess, step, critf, itermax, x)

!**********************************************************************
! June 27, 2004 Jing
!
! General form of bi-section method to compute (kp, gw(a'))
! can be used with any number of binding constraint --- Yan
!
!
! Update policy function using derivative vk and vb of value function
!
! Enforcement constraints are binding except for top 2 a'
!
! Input : xguess: kp       
!		  step:   parameter for steps
!         critf:  criterion for convergence
!         itermax: maximum iteration number
!         neq:    number of a' for this a
!
! Output: x:      optimal kp
!         ggg:    public variable which recordes gw(a') for p(a'|a)>0.0 and enf constraint is not binding
!                 and gama(a') for a' such that enf constraints are binding
!
! f(i)      =   lhs(i) + rhs(i)
! 
! f(end)    =   (1-1/R)(1-1/R*sum(p(a'|a)*(1-delta+alpha*a'*kp**(alpha-1)))) 
!              - sum( gama(a') * duaut(a',kp)/dkp )
! 
! lhs(i)    =   -(1-1/R)*beta*p(a'|a)* c'(h)/(1-beta)  
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a'))               if icon(i) == 0         
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a')) - gamm(a')     if icon(i) == 1         
!
! Need to take care of the case where h>hmax (technically need (h(1-sigma)+1)>0.0)
! Consider using taylor rule to extend the function c'(h) as
!   c'(h) = c'(hmax) + c''(hmax)* (h - hmax)
!
! Base taking care of the case where wp<wmin or wp<wmax
!
! Subroutine used:
!	base(wp, valw, indw):
!       input  : wp
!       output : indw (index of W interval which wp falls into)
!                valw (weight of each corner points)
!
!***********************************************************************

USE penfvar
implicit none

integer, intent(in)            :: itermax
real, intent(in)               :: xguess, critf, step
real, intent(out)              :: x

integer                        :: mmm, i, inx, iters, ibsec, jj,ibsecw,mm
real                           :: kp, h, temp0, temp1, temp2, temp3, temp, fnorm1, fval, xmin, xmax, gsign
real                           :: vratio, kp1

real, dimension(lp)           :: rhs,lhs
real, dimension(la)            :: uaut, duaut
real, dimension(lp)           :: g, gamma1, dg
real, dimension(2)             :: valw
integer, dimension(2)          :: indw

real:: fvalw,stepw

external base, basevw, dbsint, euler0, fvalue0

ibsecw=0
kp               = xguess
xmax             = gk1(ia, iw+1)
iters            = 1
ibsec            = 0
g                = 0.0
dg               = 0.0
gamma1           = 0.0
kp1              = 0.0
do
	call intaut(kp, uaut, duaut)
		
	if (ienf>=1) then
		do i = lp-ienf+1, lp
				g(i)     = uaut(ix(i))
				dg(i)    = duaut(ix(i))
		end do
	end if
					
	! if at least one enforcement constraint NOT binding
	if (ienf<lp) then

		dg(1:lp-ienf)            = 0.0

		! First compute g(1)
		if (iters == 1) then 
			if (iw == lw) then
				g(1) = ggg(1)
			else
				if (gw1(ipp(ia), iw+1) < 0.0) then
					g(1)            = gw1(ipp(ia), iw+1)
				else
					g(1)            = gw1(ipp(ia), iw+1)
				end if
			end if
		end if
		
		ggg   = g
		
		if (ibsecw==1) then
			stepw = 1.05
			call bisecw(g(1),fvalw,stepw,crt*10,itermax)
		else
			call dneqnf(euler0, crt*10, 1, 2000, g(1), g(1), fnorm1)
		end if
		
		inx     = ipp(ia)
		if (ilinear == 1) then
			! Linear interpolation
			call base(g(1), valw, indw)	
			temp1   = 0.0
			do mmm = 1, 2       !k
				temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
			end do
		else
			call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
			call basevw(g(1), temp1)
		end if
		
		vvg   = temp1
		
		! if more than 2 enforcement constrains NOT binding
		if (lp-ienf>=2) then
			do i=2,lp-ienf
				do jj=1,lw
					if (vw(jpp(inx+i-1),jj)>=vvg) exit
				end do
		
				if (jj>1) then
					vratio = (vw(jpp(inx+i-1),jj)-vvg)/(vvg-vw(jpp(inx+i-1),jj-1))
					g(i)   = (W(jj)+vratio*W(jj-1))/(1+vratio)
				else
					vratio = (vw(jpp(inx+i-1),2)-vvg)/(vvg-vw(jpp(inx+i-1),1))
					g(i)   = (W(2)+vratio*W(1))/(1+vratio)
				end if
			end do
		end if ! end if more than 2 enforcement constrains NOT binding
	end if !end if at least one enforcement constraint NOT binding
	
	! calculate current period utility level
	temp  = 0.0
	do i = 1, lp
		inx     = ipp(ia) + i - 1
		temp    = temp + pp(inx)* g(i)
	end do
	h           = (W(iw) - beta * temp)/(1.0 - beta)

	! The first derivative of C at h		
	if (h <= hmax) then
		temp0   = dcc(h)
	else
		temp0   = dcc(hmax) + ddcc(hmax) * (h-hmax)       ! Extend the derivative of inverse utility function
	end if
	
	do i = 1, lp
		inx     = ipp(ia) + i - 1
		lhs(i)  = -(1.0 - 1.0/R) * beta * pp(inx) * temp0 /(1.0 - beta) 
	
		if (ilinear == 1) then
			! Linear interpolation
			call base(g(i), valw, indw)

			temp1   = 0.0
			do mmm = 1, 2       !k
				temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
			end do
		
		else
			call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
			call basevw(g(i), temp1)
		end if
			
		rhs(i)    = 1.0/R * pp(inx) * temp1
		gamma1(i) = lhs(i) + rhs(i)
	end do
	
	if (ienf<lp) gamma1(1:lp-ienf) = 0.0
	
	temp2     = 0.0
	temp3     = 0.0
	
	do i = 1, lp
		inx       = ipp(ia) + i - 1
		temp2     = temp2 + pp(inx) * (1.0-delta+alpha*A(ix(i))*kp**(alpha-1)) 
		temp3     = temp3 + coaut*gamma1(i) * dg(i)
	end do
	fval           = (1.0-1.0/R)*(1.0-1.0/R * temp2) + temp3
	
	gsign = 0.0
	do i = lp-ienf+1, lp
		if (gamma1(i) < gsign) then
			gsign = gamma1(i)
		end if
	end do		
	
	if (ienf<lp) then
		if (iwtemp ==1) write(30,12) fval, kp, g(1:lp-ienf), gamma1(lp-ienf+1:lp), h, hmax
	else
		if (iwtemp ==1) write(30,12) fval, kp, gamma1(1:lp), h, hmax
	end if	
	
	if ((abs(fval)<critf .and. gsign > gammac)) then ! .or. (abs(kp-kp1)<1.0e-9)) then
		
		x          = kp
		if (ienf<lp) then
			ggg(1:lp-ienf)      = g(1:lp-ienf)
			if (ienf>=1) then
				do mm=lp-ienf+1,lp
!					if (gamma1(mm)<0.0) then
!						ggg(mm) = gamma1(mm)-gammac
!					else
!						ggg(mm) = gamma1(mm)
!					end if
					ggg(mm) = gamma1(mm)
					
				end do
			end if
		else
			do mm=1,lp
!				if (gamma1(mm)<0.0) then
!					ggg(mm)   = gamma1(mm)-gammac
!				else
!					ggg(mm)   = gamma1(mm)
!				end if
				ggg(mm)   = gamma1(mm)
			end do
		end if
		exit
	elseif (iters > itermax) then
		print *, 'Increase the maximum iteration number'
		stop
	else
		if (fval >= 0.0) then
			xmax  = kp
			kp1   = kp
			if (ibsec == 0) then
				!write(30,*) kp, step, xmax 
				kp = step * xmax
			else
				kp = (xmax + xmin)/2.0
				!write(30,*) kp, xmin, xmax 
				!if (xmax-xmin<=critf*(1+abs(xmin)+abs(xmax))) exit
			end if
		else
			xmin  = kp
			kp1   = kp
			ibsec = 1
			kp    = (xmax + xmin)/2.0
			!if (xmax-xmin<=critf*(1+abs(xmin)+abs(xmax))) exit
		end if
	end if
	iters = iters + 1
end do

12 format(12E18.5)

End subroutine bisec0