Subroutine bisecw(g1,fval,step,critf,itermax)

!**********************************************************************
! 
! Given kp to compute ga'(1) using bi-section method
! 
! f(i)      =   lhs(i) + rhs(i)
! lhs(i)    =   -(1-1/R)*beta*p(a'|a)* c'(h)/(1-beta)  
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a'))    
!***********************************************************************

USE penfvar
implicit none

real, intent(inout)    :: g1
real, intent(out)      :: fval
real, intent(in)       :: step,critf
integer, intent(in)     :: itermax

real, dimension(lp)            :: gtemp
integer                        :: mmm, i, inx, jj,ibsecp,ibsecn,iters
real                           :: h, temp0, temp1, temp, fnorm1, vratio

real                           :: rhs,lhs,gmax,gmin,pg1
real, dimension(2)             :: valw
integer, dimension(2)          :: indw

external base, basevw, dbsint, fvalue0

inx             = ipp(ia)

iters  = 1
ibsecp  = 0
ibsecn  = 0
pg1     = 2.0
do 
	gtemp(1)        = g1

	if (ilinear == 1) then
		! Linear interpolation
		call base(gtemp(1), valw, indw)	
		temp1   = 0.0
		do mmm = 1, 2       !k
			temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
		end do
	else
		call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
		call basevw(gtemp(1), temp1)
	end if
	vvg             = temp1
	
	if (lp-ienf>=2) then
		do i=2, lp-ienf
			do jj = 1,lw
				if (vw(jpp(inx+i-1),jj) >= vvg) exit
			end do
	
			if (jj > 1) then
				vratio     = (vw(jpp(inx+i-1),jj) - vvg)/(vvg-vw(jpp(inx+i-1),jj-1))
				gtemp(i)   = (W(jj)+vratio*W(jj-1))/(1.0+vratio)
			else
				vratio     = (vw(jpp(inx+i-1),2)-vvg)/(vvg-vw(jpp(inx+i-1),1))
				gtemp(i)   = (W(2)+vratio*W(1))/(1.0+vratio)
			end if
		end do
	end if
	
	if (ienf>=1) gtemp(lp-ienf+1:lp)     = ggg(lp-ienf+1:lp)
	
	! calculate current period utility level
	temp  = 0.0
	do i = 1, lp
		inx     = ipp(ia) + i - 1
		temp    = temp + pp(inx)* gtemp(i)
	end do
	h   = (W(iw) - beta * temp)/(1.0 - beta)
	
	! The first derivative of C at h		
	if (h <= hmax) then
		temp0   = dcc(h)
	else
		temp0   = dcc(hmax) + ddcc(hmax) * (h-hmax)       ! Extend the derivative of inverse utility function
	end if
		
	inx     = ipp(ia)
	lhs     = -(1.0 - 1.0/R) * beta * pp(inx) * temp0 /(1.0 - beta) 
	rhs     = 1.0/R * pp(inx) * temp1
	
	fval    = lhs + rhs

!	if (ia ==2 .and. iw == 135) then
!		write(30,*) 'vvg: ', vvg, 'g:', gtemp, 'h: ', h, 'fval: ', fval
!	end if
!	
	
	if (abs(fval)<critf .or. abs(pg1-g1)<1.0e-9) then
		if (abs(fval)>=critf) print *, ia, iw, fval
		exit
	elseif (iters>itermax) then
		print *, 'Maximum interation number arrives at bisecw', ia, iw, fval
		exit	
	else
		pg1 = g1
		if (fval>=0.0) then
			gmax = g1
			ibsecp = 1
			if (ibsecn == 0) then
				g1 = 1.05*gmax
			else
				g1 = (gmax + gmin)/2.0
			end if
		else
			gmin = g1
			ibsecn = 1
			if (ibsecp ==0) then
				g1 = 0.95*gmin
			else
				g1    = (gmax + gmin)/2.0
			end if
		end if
	end if

	iters = iters + 1
end do
			
End subroutine bisecw
