subroutine distbase(wp,valw,indw, kp, valk, indk)

!***************************************************************
! Subroutine for computing transition matrix
! Find the interval index that (kp, bp) falls into
! Calculate the weight (probability) of each corner point
!
! Different from base.f90 is the definition of weight for 
!    the points outside the set K and B
! base.f90 still uses usual method.
! distbase.f90 needs all the weights be nonnegative.
! So when kp>kmax, let valk(2)=1 and valk(1)=0
!
! June 16, 2004
! Jing and Yan
!***************************************************************
!***************************************************************
! Subroutine for linear interpolation
! Find the interval index that wp falls into
! Calculate the weight of each edge point
!
! In distbase, we need all the weights be nonnegative.
!
! June 16, 2004
! Jing and Yan
!***************************************************************

USE penfvar

implicit none


real,intent(in)::                     wp, kp
real,intent(out),dimension(2)::       valw, valk
integer, intent(out),dimension(2)::   indw, indk

integer::                             bbb, kkk
logical::                             elemw, elemk

elemw = .false.
bbb   = 1

do while ((.not. elemw) .and. (bbb <= lw-1))
	elemw=((W(bbb) <= wp) .and. (W(bbb+1) >= wp))
	bbb=bbb+1
end do

select case(elemw)

case(.true.)
	valw(2)       = (wp-W(bbb-1))/(W(bbb)-W(bbb-1))
	valw(1)       = (W(bbb)-wp)/(W(bbb)-W(bbb-1))
	indw(2)       = bbb
	indw(1)       = bbb-1

case(.false.)

	if (wp>W(lw)) then
		valw(2)   = 1.0
		valw(1)   = 0.0
		indw(2)   = lw
		indw(1)   = lw-1
	end if
	if (wp<W(1)) then
		valw(2)   = 0.0
		valw(1)   = 1.0
		indw(2)   = 2
		indw(1)   = 1
	end if
end select

elemk = .false.
kkk   = 1

do while ((.not. elemk) .and. (kkk <= lk-1))
	elemk = ((K(kkk) <= kp) .and. (K(kkk+1) >= kp))
	kkk   = kkk+1
end do

select case(elemk)

case(.true.)
	valk(2)       = (kp-K(kkk-1))/(K(kkk)-K(kkk-1))
	valk(1)       = (K(kkk)-kp)/(K(kkk)-K(kkk-1))
	indk(2)       = kkk
	indk(1)       = kkk-1

case(.false.)

	if (kp>K(lk)) then
		valk(2)   = 1.0
		valk(1)   = 0.0
		indk(2)   = lk
		indk(1)   = lk-1
	end if
	if (kp<K(1)) then
		valk(2)   = 0.0
		valk(1)   = 1.0
		indk(2)   = 2
		indk(1)   = 1
	end if
end select

return

end subroutine distbase