Subroutine euler0(xguess,fval,nx)

!**********************************************************************
! Oct 12, 2007 Jing
!
! Given kp to compute ga'(1) to satisfy the euler
! where the other two a' are binding
! f(i)      =   lhs(i) + rhs(i)
! lhs(i)    =   -(1-1/R)*beta*p(a'|a)* c'(h)/(1-beta)  
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a'))    
!***********************************************************************

USE penfvar
implicit none

integer, intent(in)            :: nx
real, intent(in),dimension(nx) :: xguess
real, intent(out),dimension(nx):: fval

real, dimension(lp)            :: gtemp
integer                        :: mmm, i, inx, jj
real                           :: h, temp0, temp1, temp, fnorm1, vratio

real                           :: rhs,lhs
real, dimension(2)             :: valw
integer, dimension(2)          :: indw

external base, basevw, dbsint, fvalue0

gtemp(1)        = xguess(1)


inx             = ipp(ia)
if (ilinear == 1) then
	! Linear interpolation
	call base(gtemp(1), valw, indw)	
	temp1   = 0.0
	do mmm = 1, 2       !k
		temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
	end do
else
	call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
	call basevw(gtemp(1), temp1)
end if
vvg             = temp1

if (lp-ienf>=2) then
	do i=2, lp-ienf
		do jj = 1,lw
			if (vw(jpp(inx+i-1),jj) >= vvg) exit
		end do

		if (jj > 1) then
			vratio     = (vw(jpp(inx+i-1),jj) - vvg)/(vvg-vw(jpp(inx+i-1),jj-1))
			gtemp(i)   = (W(jj)+vratio*W(jj-1))/(1.0+vratio)
		else
			vratio     = (vw(jpp(inx+i-1),2)-vvg)/(vvg-vw(jpp(inx+i-1),1))
			gtemp(i)   = (W(2)+vratio*W(1))/(1.0+vratio)
		end if
	end do
end if

if (ienf>=1) gtemp(lp-ienf+1:lp)     = ggg(lp-ienf+1:lp)

! calculate current period utility level
temp  = 0.0
do i = 1, lp
	inx     = ipp(ia) + i - 1
	temp    = temp + pp(inx)* gtemp(i)
end do
h   = (W(iw) - beta * temp)/(1.0 - beta)

! The first derivative of C at h		
if (h <= hmax) then
	temp0   = dcc(h)
else
	temp0   = dcc(hmax) + ddcc(hmax) * (h-hmax)       ! Extend the derivative of inverse utility function
end if
	
inx     = ipp(ia)
lhs     = -(1.0 - 1.0/R) * beta * pp(inx) * temp0 /(1.0 - beta) 
rhs     = 1.0/R * pp(inx) * temp1

fval    = lhs + rhs

if (ia ==2 .and. iw == 16) then
	write(30,*) 'vvg: ', vvg, 'g:', gtemp, 'h: ', h, 'fval: ', fval
end if

End subroutine euler0
