Subroutine fvalue(xguess,fval,nx)

!**********************************************************************
! June 17, 2004 Jing
!
! Update policy function using derivative vk and vb of value function
!
!          Enforcement constraints are not binding for all a'
!
! Input : xguess (gw(a') for all a' such that p(a'|a)>0.0)
!         nx     (dimension of unknowns)
! Output: fval   (dimension of nx)
!
! f(i)    =   lhs(i) + rhs(1)
! lhs(i)  =   -(1-1/R)*beta*p(a'|a)* c'(h)/(1-beta)
! rhs(i)  =   1/R * p(a'|a)*vw(a',gw(a'))! 
!
! Need to take care of the case where h>hmax (technically need (h(1-sigma)+1)>0.0)
! Consider using taylor rule to extend the function c'(h) as
!   c'(h) = c'(hmax) + c''(hmax)* (h - hmax)
!
! Base taking care of the case where wp<wmin or wp<wmax
!
! Subroutine used:
!	base(wp, valw, indw):
!       input  : wp
!       output : indw (index of W interval which wp falls into)
!                valw (weight of each corner points)
!
!***********************************************************************

USE penfvar
implicit none

integer, intent(in)            :: nx
real, intent(in),dimension(nx) :: xguess
real, intent(out),dimension(nx):: fval

integer                        :: mmm, i, inx
real                           :: h, temp0, temp1, temp

real, dimension(nx)            :: rhs,lhs
real, dimension(2)             :: valw
integer, dimension(2)          :: indw

external base, basevw, dbsint

temp        =0.0
do i = 1, nx
	inx     = ipp(ia) + i - 1
	temp    = temp + pp(inx)* xguess(i)
end do

h           = (W(iw)-beta*temp)/(1.0 - beta)
		
if (h <= hmax) then
	temp0   = dcc(h)
else
	temp0   = dcc(hmax) + ddcc(hmax) * (h-hmax)       ! Extend the derivative of inverse utility function
end if

do i = 1, nx
	inx     = ipp(ia) + i - 1
	lhs(i)  = -(1.0 - 1.0/R) * beta * pp(inx) * temp0 /(1.0 - beta) 

	if (ilinear == 1) then
		! Linear interpolation
		call base(xguess(i), valw, indw)
		temp1   = 0.0
		do mmm = 1, 2       !k
			temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
		end do
	else
		call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
		call basevw(xguess(i), temp1)
	end if

	rhs(i)    = 1.0/R * pp(inx) * temp1
end do

fval          = lhs + rhs

!if (ia==2 .and. iw==310) 
write(30,10) sum(abs(fval)), xguess

10 format(F9.4, 7F9.4)

End subroutine fvalue
