Subroutine fvalue1(xguess,fval,nx)

!**********************************************************************
! June 27, 2004 Jing
!
! Update policy function using derivative vk and vb of value function
!
! Enforcement constraints are binding for one top a'
!
! Input : xguess [gw(a')   for a' such that p(a'|a)>0.0 and enf constraint is not binding
!                [gama(a') for a' such that enf constraints are binding
!                [kp       
!
!         nx     (dimension of unknowns)
!
! Output: fval   (dimension of nx)
!
! f(i)      =   lhs(i) + rhs(i)
! 
! f(end)    =   (1-1/R)(1-1/R*sum(p(a'|a)*(1-delta+alpha*a'*kp**(alpha-1)))) 
!                - sum( gama(a') * duaut(a',kp)/dkp )
! 
! lhs(i)    =   -(1-1/R)*beta*p(a'|a)* c'(h)/(1-beta)  
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a'))               if icon(i) == 0         
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a'))+ gamm(a')     if icon(i) == 1         
!
! Need to take care of the case where h>hmax (technically need (h(1-sigma)+1)>0.0)
! Consider using taylor rule to extend the function c'(h) as
!   c'(h) = c'(hmax) + c''(hmax)* (h - hmax)
!
! Base taking care of the case where wp<wmin or wp<wmax
!
! Subroutine used:
!	base(wp, valw, indw):
!       input  : wp
!       output : indw (index of W interval which wp falls into)
!                valw (weight of each corner points)
!
!***********************************************************************

USE penfvar
implicit none

integer, intent(in)            :: nx
real, intent(in),dimension(nx) :: xguess
real, intent(out),dimension(nx):: fval

integer                        :: mmm, i, inx
real                           :: kp, h, temp0, temp1, temp2, temp3, temp

real, dimension(nx)            :: rhs,lhs
real, dimension(la)            :: uaut, duaut
real, dimension(nx-1)          :: g, gamma1, puaut, pduaut
real, dimension(2)             :: valw
integer, dimension(2)          :: indw

external base, basevw, dbsint

kp          = xguess(nx)
call intaut(kp, uaut, duaut)

do i = 1, nx-1
	puaut(i)       = uaut(ix(i))
	pduaut(i)      = duaut(ix(i))
end do

if (nx == 2) then
	gamma1(nx-1)   = xguess(nx-1)
	g(nx-1)        = coaut*puaut(nx-1)
else
	gamma1(nx-1)   = xguess(nx-1)
	gamma1(1:nx-2) = 0.0
	g(1:nx-2)      = xguess(1:nx-2)
	g(nx-1)        = coaut*puaut(nx-1)
end if

!if (kp < 1.0e-5) then
if (kp < kmin .or. gamma1(nx-1)<0.0) then
	!print *, 'kp is out of range'
	!print *, 'kp=  ', kp
	fval   = 1.0e5
else

	! calculate current period utility level
	temp        =0.0
	do i = 1, nx-1
		inx     = ipp(ia) + i - 1
		temp    = temp + pp(inx)* g(i)
	end do
	h           = (W(iw) - beta * temp)/(1.0 - beta)

	! The first derivative of C at h		
	if (h <= hmax) then
		temp0   = dcc(h)
	else
		temp0   = dcc(hmax) + ddcc(hmax) * (h-hmax)       ! Extend the derivative of inverse utility function
	end if

	temp2       = 0.0
	temp3       = 0.0
	do i = 1, nx-1
		inx     = ipp(ia) + i - 1
		lhs(i)  = -(1.0 - 1.0/R) * beta * pp(inx) * temp0 /(1.0 - beta) 

		if (ilinear == 1) then
			! Linear interpolation
			call base(g(i), valw, indw)
			temp1   = 0.0
			do mmm = 1, 2       !k
				temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
			end do
		else
			call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
			call basevw(g(i), temp1)
		end if
	
		rhs(i)    = 1.0/R * pp(inx) * temp1 - gamma1(i)
		temp2     = temp2 + pp(inx) * (1.0-delta+alpha*A(ix(i))*kp**(alpha-1)) 
		temp3     = temp3 + coaut * gamma1(i) * pduaut(i)
	end do

	lhs(nx)       = (1.0-1.0/R)*(1.0-1.0/R * temp2)
	rhs(nx)       = temp3

	fval          = lhs + rhs
end if

if (iwtemp ==1) write(30,12) sum(abs(fval)), xguess

12 format(F9.4, 7F9.4)

End subroutine fvalue1


