Subroutine fvaluek(xguess, fval, nx)

!**********************************************************************
! June 27, 2004 Jing
!
! Compute k and all the enforcement constraint binding!!
!
! Update policy function using derivative vk and vb of value function
!
! Enforcement constraints are binding except for top 2 a'
!
! Input : xguess: kp       
!		  step:   parameter for steps
!         critf:  criterion for convergence
!         itermax: maximum iteration number
!         neq:    number of a' for this a
!
! Output: x:      optimal kp
!         ggg:    public variable which recordes gw(a') for p(a'|a)>0.0 and enf constraint is not binding
!                 and gama(a') for a' such that enf constraints are binding
!
! f(i)      =   lhs(i) + rhs(i)
! 
! f(end)    =   (1-1/R)(1-1/R*sum(p(a'|a)*(1-delta+alpha*a'*kp**(alpha-1)))) 
!              - sum( gama(a') * duaut(a',kp)/dkp )
! 
! lhs(i)    =   -(1-1/R)*beta*p(a'|a)* c'(h)/(1-beta)  
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a'))               if icon(i) == 0         
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a')) - gamm(a')     if icon(i) == 1         
!
! Need to take care of the case where h>hmax (technically need (h(1-sigma)+1)>0.0)
! Consider using taylor rule to extend the function c'(h) as
!   c'(h) = c'(hmax) + c''(hmax)* (h - hmax)
!
! Base taking care of the case where wp<wmin or wp<wmax
!
! Subroutine used:
!	base(wp, valw, indw):
!       input  : wp
!       output : indw (index of W interval which wp falls into)
!                valw (weight of each corner points)
!
!***********************************************************************

USE penfvar
implicit none

integer, intent(in)            :: nx
real, intent(in),dimension(nx) :: xguess
real, intent(out),dimension(nx):: fval

integer                        :: mmm, i, inx, iters, ibsec, jj,ibsecw,mm,gsign
real                           :: kp, h, temp0, temp1, temp2, temp3, temp
real                           :: vratio

real                           :: rhs(lp),lhs(lp),lhs1,rhs1
real, dimension(la)            :: uaut, duaut
real, dimension(lp)            :: g, gamma1, dg
real, dimension(2)             :: valw
integer, dimension(2)          :: indw

real:: fvalw,stepw
real:: kk1, kk2, mkk, gridk

external base, basevw, dbsint, euler0, fvalue0

kp        = xguess(1)
if (kp<kmin) then
	fval = 1.0e5
else

	! autarky utility
	call intaut(kp, uaut, duaut)
!	
!	if (ia==8 .and. iw==28) then
!		kk1=3.6e-3
!		kk2=4.5e-3
!		gridk=(kk2-kk1)/100.0
!		print *, 'hiihihihi'
!		do i=1,100
!			mkk=kk1+gridk*(i-1)
!			call intaut(mkk, uaut, duaut)
!			print *, mkk, duaut
!		end do
!	end if
		
	!future welfare for Binding constraint
	do i = 1, lp
		g(i)     = uaut(ix(i))
		dg(i)    = duaut(ix(i))
	end do
			
	! calculate current period utility level
	temp  = 0.0
	do i = 1, lp
		inx     = ipp(ia) + i - 1
		temp    = temp + pp(inx)* g(i)
	end do
	h           = (W(iw) - beta * temp)/(1.0 - beta)
	
	! The first derivative of C at h		
	if (h <= hmax) then
		temp0   = dcc(h)
	else
		temp0   = dcc(hmax) + ddcc(hmax) * (h-hmax)       ! Extend the derivative of inverse utility function
	end if
	
	! Compute gamma for Binding state
	do i = 1, lp
		inx     = ipp(ia) + i - 1
		lhs(i)  = -(1.0 - 1.0/R) * beta * pp(inx) * temp0 /(1.0 - beta) 
	
		if (ilinear == 1) then
			! Linear interpolation
			call base(g(i), valw, indw)
	
			temp1   = 0.0
			do mmm = 1, 2       !k
				temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
			end do
		
		else
			call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
			call basevw(g(i), temp1)
		end if
					
	
		rhs(i)    = 1.0/R * pp(inx) * temp1
		gamma1(i) = lhs(i) + rhs(i)
		ggg(i)    = gamma1(i)
		
	end do
	
	! fvalue for kp
	temp2   = 0.0
	temp3   = 0.0
	do i = 1, lp
		inx       = ipp(ia) + i - 1
		temp2     = temp2 + pp(inx) * (1.0-delta+alpha*A(ix(i))*kp**(alpha-1)) 
		temp3     = temp3 + coaut*gamma1(i) * dg(i)
	end do
	
	fv1=(1.0-1.0/R)*(1.0-1.0/R * temp2)
	fv2=temp3
	fval(1)       = (1.0-1.0/R)*(1.0-1.0/R * temp2) + temp3
	
	gsign = 0
	do i = 1, lp
		if (gamma1(i) < 0.0) then
			gsign = 1
			exit
		end if
	end do
	
	
	!if (gsign==1) fval =1.0e5
end if
	
if (iwtemp ==1) write(30,12) sum(abs(fval)), xguess,ggg(1:lp)

12 format(12E18.5)

End subroutine fvaluek