Subroutine fvaluekw(xguess, fval, nx)

!**********************************************************************
! June 27, 2004 Jing
!
! Compute k and g1: at least one enforcement constraint not binding!!
!
! Update policy function using derivative vk and vb of value function
!
! Enforcement constraints are binding except for top 2 a'
!
! Input : xguess: kp       
!		  step:   parameter for steps
!         critf:  criterion for convergence
!         itermax: maximum iteration number
!         neq:    number of a' for this a
!
! Output: x:      optimal kp
!         ggg:    public variable which recordes gw(a') for p(a'|a)>0.0 and enf constraint is not binding
!                 and gama(a') for a' such that enf constraints are binding
!
! f(i)      =   lhs(i) + rhs(i)
! 
! f(end)    =   (1-1/R)(1-1/R*sum(p(a'|a)*(1-delta+alpha*a'*kp**(alpha-1)))) 
!              - sum( gama(a') * duaut(a',kp)/dkp )
! 
! lhs(i)    =   -(1-1/R)*beta*p(a'|a)* c'(h)/(1-beta)  
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a'))               if icon(i) == 0         
! rhs(i)    =   1/R * p(a'|a)*vw(a',gw(a')) - gamm(a')     if icon(i) == 1         
!
! Need to take care of the case where h>hmax (technically need (h(1-sigma)+1)>0.0)
! Consider using taylor rule to extend the function c'(h) as
!   c'(h) = c'(hmax) + c''(hmax)* (h - hmax)
!
! Base taking care of the case where wp<wmin or wp<wmax
!
! Subroutine used:
!	base(wp, valw, indw):
!       input  : wp
!       output : indw (index of W interval which wp falls into)
!                valw (weight of each corner points)
!
!***********************************************************************

USE penfvar
implicit none

integer, intent(in)            :: nx
real, intent(in),dimension(nx) :: xguess
real, intent(out),dimension(nx):: fval

integer                        :: mmm, i, inx, iters, ibsec, jj,ibsecw,mm,gsign
real                           :: kp, h, temp0, temp1, temp2, temp3, temp
real                           :: vratio

real                           :: rhs(lp),lhs(lp),lhs1,rhs1
real, dimension(la)            :: uaut, duaut
real, dimension(lp)            :: g, gamma1, dg
real, dimension(2)             :: valw
integer, dimension(2)          :: indw

real:: kk1, kk2, mkk, gridk

real:: fvalw,stepw

external base, basevw, dbsint, euler0, fvalue0

kp        = xguess(1)

if (kp<kmin) then
	fval = 1.0e5
else
	g(1)      = xguess(2)
	
	inx     = ipp(ia)
	if (ilinear == 1) then
		! Linear interpolation
		call base(g(1), valw, indw)	
		temp1   = 0.0
		do mmm = 1, 2       !k
			temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
		end do
	else
		call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
		call basevw(g(1), temp1)
	end if
	ggg(1) = g(1)
	vvg   = temp1
	
	! autarky utility
	call intaut(kp, uaut, duaut)
	
!	if (ia==1 .and. iw==100) then
!		kk1=kmin
!		kk2=1.0e-3
!		gridk=(kk2-kk1)/400.0
!		print *, 'hiihihihi'
!		do i=1,400
!			mkk=kk1+gridk*(i-1)
!			call intaut(mkk, uaut, duaut)
!			print *, mkk, uaut
!		end do
!		
!	end if
!			
	
	! future welfare for NON-Binding states
	dg(1:lp-ienf)   = 0.0
	
	! if more than 2 enforcement constrains NOT binding
	if (lp-ienf>=2) then
		do i=2,lp-ienf
			do jj=1,lw
				if (vw(jpp(inx+i-1),jj)>=vvg) exit
			end do
	
			if (jj>1) then
				vratio = (vw(jpp(inx+i-1),jj)-vvg)/(vvg-vw(jpp(inx+i-1),jj-1))
				g(i)   = (W(jj)+vratio*W(jj-1))/(1+vratio)
			else
				vratio = (vw(jpp(inx+i-1),2)-vvg)/(vvg-vw(jpp(inx+i-1),1))
				g(i)   = (W(2)+vratio*W(1))/(1+vratio)
			end if
			ggg(i) = g(i)
		end do
	end if ! end if more than 2 enforcement constrains NOT binding
	
	!future welfare for Binding constraint
	if (ienf>=1) then
		do i = lp-ienf+1, lp
			g(i)     = uaut(ix(i))
			dg(i)    = duaut(ix(i))
		end do
	end if
			
	! calculate current period utility level
	temp  = 0.0
	do i = 1, lp
		inx     = ipp(ia) + i - 1
		temp    = temp + pp(inx)* g(i)
	end do
	h           = (W(iw) - beta * temp)/(1.0 - beta)
	
	! The first derivative of C at h		
	if (h <= hmax) then
		temp0   = dcc(h)
	else
		temp0   = dcc(hmax) + ddcc(hmax) * (h-hmax)       ! Extend the derivative of inverse utility function
	end if
	
	! Compute gamma for Binding state
	gamma1 = 0.0
	if (ienf>=1) then
		do i = lp-ienf+1, lp
			inx     = ipp(ia) + i - 1
			lhs(i)  = -(1.0 - 1.0/R) * beta * pp(inx) * temp0 /(1.0 - beta) 
			
			if (ilinear == 1) then
				! Linear interpolation
				call base(g(i), valw, indw)
	
				temp1   = 0.0
				do mmm = 1, 2       !k
					temp1 = temp1+vw(jpp(inx),indw(mmm))*valw(mmm)
				end do		
			else
				call dbsint(lw,W, vw(jpp(inx),:),kxord,wknot,vwcoef)
				call basevw(g(i), temp1)
			end if
					
			rhs(i)    = 1.0/R * pp(inx) * temp1
			gamma1(i) = lhs(i) + rhs(i)
			ggg(i) = gamma1(i)
		end do
	end if
	
	! fvalue for gw1
	inx     = ipp(ia)
	lhs1    = -(1.0 - 1.0/R) * beta * pp(inx) * temp0 /(1.0 - beta) 
	rhs1    = 1.0/R * pp(inx) * vvg
	fval(2) = lhs1 + rhs1
	
	! fvalue for kp
	temp2   = 0.0
	temp3   = 0.0
	do i = 1, lp
		inx       = ipp(ia) + i - 1
		temp2     = temp2 + pp(inx) * (1.0-delta+alpha*A(ix(i))*kp**(alpha-1)) 
		temp3     = temp3 + coaut*gamma1(i) * dg(i)
	end do
	fval(1)       = (1.0-1.0/R)*(1.0-1.0/R * temp2) + temp3
	
	gsign = 0
	if (ienf>=1) then
		do i = lp-ienf+1, lp
			if (gamma1(i) < 0.0) then
				gsign = 1
				exit
			end if
		end do
	end if
	
	!if (gsign==1) fval =1.0e5
	
	if (iwtemp ==1) then
		 write(30,12) sum(abs(fval)), xguess(1), ggg(1:lp)
	end if
end if
	

12 format(12E18.5)

End subroutine fvaluekw