PROGRAM penforce
!
! Consider return to markets case
! See the word file enforcemore.doc
!
!*******************************************************

USE penfvar

IMPLICIT NONE

integer                             :: i, j, ii, ibeta,kxord1
real                                :: disp1, disp2, disp3, disp4, ttv,ttv1, ttk, ttk1, ttw, ttw1, ttvw, ttvw1
real                                :: disp5, ttv2, ttv3,aashare
real :: kk1, kk2, gridk, mkk,uaut(la), duaut(la)

EXTERNAL autarkyU, fvalue, fvalue1, guess, initial, policy, updatedrv, base, distbase, writeout, vwbzero, excessd, basevw
external dbsnak, dbsint, dneqnf, dbsval, dbsder, intaut


talpha = 1.0/(1.0-alpha)
hmax   = 1.0/(sigma-1.0) - eps   

!*****************************************************************************
! Initializing the shocks and state space
call initial                ! Read in shock and put P into CSR format
							! Initialize K and W space and autarky utility

! Initial guess for policy function gk, gw and derivative of value function vw
allocate(gw(indexp,lw),gw1(indexp,lw), oddgw(indexp, lw), evengw(indexp, lw))

call guess                  ! Need R to update gk

!***************************************************************************
! Start to solve the problem
rewind(200)
print *, 'Start solving complete market with enforcement constraint'
write(200,*) 'Start solving complete market with enforcement constraint'
beta   = betamax - hbeta
ibeta  = 1
do ! iterate over beta to clear market
	if (beta <= betamin) exit
	print *, 'beta:   ', beta
	write(200,*)  'beta:   ', beta
!	icount  = 1
	rou  = roumin + hrou
	
	do  ! iterate over rho-- return probability to market
		if (rou > roumax) then
			 rou = rou- hrou
			 exit
		end if

		print *, 'rou:  ', rou
		write(200, *) 'rou:  ', rou
		
		
		disp4   = 2.0
		disp5   = 2.0
		autup   = autu
		tvwp    = vwp
		iter      = 1
		do !Iterate over v on the right hand side (disp4) and w on the left hand side (disp5)
			
			!if (disp4 < crit4 .and. disp5 < crit4) exit
			if (disp5 < crit4 .and. disp4<crit4 .and. disp1<crit4 .and. disp2<crit4 .and. disp3<crit4) exit
			!if (disp4 < crit4  .and. disp1<crit4 .and. disp2<crit4 .and. disp3<crit4) exit
			
			! Autarky utility
			call autarkyU    ! Generate autu
			!stop
			! Interpolate the artauky utility	
			do i=1,la
				call dbsint(autlk,autk,autu(i,:),kxord,xknot,bscoef(i,:))
			end do
		
			if (icount > 200000) then
				if (mod(icount,2)==0) then
					gk  = evengk
					gw  = evengw
					vw  = evenvw
				else
					gk  = oddgk
					gw  = oddgw
					vw  = oddvw
				end if
			end if

			! First update policy function gk, gw(a')
			if (iter<100) then
				crt  = crit
			else
				crt  = crit/10
			end if
		
			! For given penalty function autu, compute the policy functions gh, gw and vw
			call policy
			!print *, 'done policy'
			! Second update derivative of value function vw
			call updatedrv
			!print *, 'done  updatedrv'
			! Convergence criterion
			ttk         = sqrt(sum((gk-gk1)**2))
			ttk1        = 1.0 + sqrt(sum(gk**2))
			disp1       = ttk / ttk1
			
			ttw         = sqrt(sum((gw-gw1)**2))
			ttw1        = 1.0 + sqrt(sum(gw**2))
			disp2       = ttw / ttw1		
			
			ttvw        = sqrt(sum((vw-vw1)**2))
			ttvw1       = 1.0 + sqrt(sum(vw**2))
			disp3       = ttvw / ttvw1		

			
			gk   =  gk1
			gw  =  gw1
			vw  =  vw1
						
			!if (iwtemp==1) then
			!	rewind(30)
			!	write(30, *) 'gk1='
			!	do i = 1, la
			!		do j = 1, lw
			!			write(30, *)  gk1(i,j)
			!		end do
			!	end do
			!	write(30, *) 'gw1='
			!	do i = 1, la
			!		do j = 1, lw
			!			do ii = ipp(i), ipp(i+1)-1
			!				write(30, *)  gw1(ii,j)
			!			end do
			!		end do
			!	end do
			!	write(30, *) 'vw1='
			!	do i = 1, la
			!		do j = 1, lw
			!			write(30, *)  vw1(i,j)
			!		end do
			!	end do
			!end if
				
			! Construct vwp
			
			call vwbzero          ! Find the utility level vwp(a,k) for each k and a when b = 0
						
			
			ttv        = sqrt(sum((autu-autup)**2))
			ttv1       = 1.0 + sqrt(sum(autu**2))
			disp4      = ttv / ttv1	
				
			ttv2       = sqrt(sum((tvwp-vwp)**2))
			ttv3       = 1.0 + sqrt(sum(vwp**2))
			disp5      = ttv2 / ttv3
			iter         = iter + 1			
				
			print *, 'Penalty function iteration: ', disp4, disp5, disp1, disp2, disp3
			write(200,*) 'Penalty function iteration: ', disp4, disp5, disp1, disp2, disp3
			

			
			aashare=0.85
			if (iter > 2)    then
				vwp1 = (1.0-aashare)*vwp + aashare*tvwp
				vwp        = vwp1
			end if 	
			autup      = autu
			tvwp       = vwp
					
			if (mod(iter,2) == 0) then
				evengk = gk
				evengw = gw
				evenvw = vw
			else
				oddgk  = gk
				oddgw  = gw
				oddvw  = vw 
			end if
		
			iter     = iter + 1
			!if (iwtemp == 1)  call writeout		
		end do                   ! End of iteration over V
		
		! Write out the output
		call writeout
		
		rou = rou + hrou
	end do ! End of iteration over the probability of returning to the markets
	
	ibeta = ibeta +1
	
	! Compute invariant distribution and excess demand
	if (iexd>0) then
		if (mod(ibeta,iexd) == 0) then
			print *,'call exd'
			call excessd
			
			if (abs(exd) < 1.0e-3  .or. exd<=0.0) then
				call writeout
				exit
			end if
		end if
	end if
!	call excessd

!	if (abs(exd) < 1.0e-3 .or. exd<0.0) then
!		call writeout
!		exit
!	end if

	!if (iwc == 1) call excessd

	! Write out the output
	call writeout		
	
	beta   = beta - hbeta
end do  ! End of iteration over beta to clear the bond marketes

!call excessd
print *,'done program!'	

END PROGRAM penforce

!*************************************************************

! NEQNJ/DNEQNJ (Single/Double precision)
! Solve a system of nonlinear equations using a modified Powell hybrid algorithm
! with a user-supplied Jacobian.

! NEQNJ (FCN, LSJAC, ERRREL, N, ITMAX, XGUESS, X, FNORM)

! FCN: User-supplied SUBROUTINE to evaluate the system of equations to be solved. 
!      CALL FCN (X, F, N)
!       X: The point at which the functions are evaluated. (Input)
!		   X should not be changed by FCN.
!       F: The computed function values at the point X. (Output)
!       N: Length of X, F. (Input)
! FCN must be declared EXTERNAL in the calling program.

! LSJAC: User-supplied SUBROUTINE to evaluate the Jacobian at a point X.
!        CALL LSJAC (N, X, FJAC)
!		N:	  Length of X. (Input)
!       X:    The point at which the function is evaluated. (Input)
!             X should not be changed by LSJAC.
!       FJAC: The computed N by N Jacobian at the point X. (Output)
! LSJAC must be declared EXTERNAL in the calling program.

! ERRREL: Stopping criterion. (Input)

! The root is accepted if the relative error between two successive approximations
! to this root is less than ERRREL.

! N:      The number of equations to be solved and the number of unknowns.(Input)
! ITMAX:  The maximum allowable number of iterations. (Input)
!         Suggested value = 200.
! XGUESS: vector(N),Input, contains the initial estimate of the root.
! X:      vector(N),Output, contains the best estimate of the root found by NEQNJ.
! FNORM:  scalar,Output

!***************************************************************

! N2QNJ/DN2QNJ

! N2QNJ (FCN, LSJAC, ERRREL, N, ITMAX, XGUESS, X,FNORM, FVEC, FJAC, R, QTF, WK)

! FVEC:   vector (N),contains the functions evaluated at X
! FJAC:   matrix(N,N),contains the orthogonal matrix Q produced by the QR factorization of the final approximate Jacobian.
! R:      vector(N * (N + 1)/2), contains the upper triangular matrix produced
!         by the QR factorization of the final approximate Jacobian. R is stored row-wise.
! QTF:    vector(N), contains the vector TRANS(Q) * FVEC.
! WK:     work vector(5 * N)

! Informational errors

! 4 1 The number of calls to FCN has exceeded ITMAX. A new initial guess may be tried.
! 4 2 ERRREL is too small. No further improvement in the approximate solution is possible.
! 4 3 The iteration has not made good progress. A new initial guess may be tried.


