Module penfvar
integer, parameter                 :: evenw  = 0
										! initial run use 1, if readin use 0
									  ! evenw  =1: using even W grid
integer, parameter                 :: readg  = 2
                                      ! readg  =0: start from precoded initial guess
									  ! readg  =1: read guesses from value function iteration
									  ! readg  =2: read guesses from previous policy function iteration
									  ! readg  =3: read guesses from complete market result
integer, parameter                 :: readaut = 1
									  ! readaut =1: read in autarky utility
									  ! readaut =0: compute autarky utility
integer, parameter                 :: ivwp  = 1
									  ! read in penalty function from previous run
									  ! 2: reconstruct vwp according to v in initial
									  
integer, parameter                 :: readd   = 0           
									  ! readd   =1: read in invariant distribution
									  ! readd   =0: start from even distribution over states
integer, parameter                 :: ilinear = 1             ! ilinear =1: using linear extrapolation for vw
									  ! ilinear =0: using taylor expansion for extrapolation
integer, parameter                 :: ismooth = 0 
									  ! ismooth =1: Smooth the derivative of cost function to w before a new iteration (updatedrv)
									  ! ismooth =0: don't smooth the derivative of cost function to w before a new iteration (updatedrv)
integer, parameter                 :: iwtemp  = 1
                                      ! iwtemp =1: write the solving process step by step for checking where the code chokes
									  ! iwtemp =0: don't write to make the code running faster 

real, parameter                    :: alpha = 0.33, sigma = 2.0, delta = 0.10, eps = 1.0e-7
real, parameter                    :: R = 1.04
real, parameter                    :: lamda = 0.0       ! output drop parameter
real, parameter                    :: coaut = 1.0
									  ! coaut = 1.0
									  ! coaut > 1.0: increase the welfare of autraky to test the model&


real, parameter                    :: betamax = 0.9497, betamin = 0.94958
real, parameter                    :: hbeta   = 0.0001
real                               :: beta 
integer                            :: iexd=0

real, parameter                    :: roumax = 0.11, roumin = 0.01      ! Probability of return to markets
real, parameter                    :: hrou   = 0.002    !rho = 0.196
real                               :: rou

real, parameter                    :: crit0  = 1.0e-7, crit=1.0e-8, crit1=1.0e-4, crit4 = 1.0e-4, pen = -1.0e10
real, parameter                    :: gammac = -1.0e-8
                                                                              ! crit0: Invariant distribution; 
									  ! crit:  dneqnf subroutine; 
									  ! crit1: policy function and value function iteration
									  ! crit4: penalty function iteration
									  ! gammac: criterion for gamma (multiplier in the euler equation)
integer, parameter                 :: lR = 2, la = 9, autlk = 400
integer, parameter                 :: lw = 90, lk = autlk
real, parameter                    :: autkmin = 1.000000000000000E-007, autkmax = 2.8
real, parameter                    :: kmin = autkmin, kmax = autkmax, wmin = -18.5, wmax = -2.5
									! step for increasing w 0.4 with 4 points

! Increase the lower bound of w if iwc = 1: if iwc=0, use lw and wmin
integer, parameter                 :: iwc = 0
										! iwc=1 if increase points or smooth
real, parameter                    :: wminl = -20.5
integer, parameter                 :: lwl   = 100

real, dimension(lwl)               :: twgrid

integer                            :: indexp, ia, iw, lp, iter, icount, indexip, ienf
real                               :: talpha, gridr, hmax, crt, exd, vvg

real, dimension(la)                :: a, Ea, kc, ggg
real, dimension(lw)                :: W
real, dimension(lk)                :: K
real, dimension(la,la)             :: p

integer, dimension(la+1)           :: ipp
integer, allocatable, dimension(:) :: jpp, icon, ix
real, allocatable, dimension(:)    :: pp

integer, dimension(la,lw)          :: indenf
real, dimension(la,lw)             :: gk, vw, gk1, vw1, cv, cv0, cvw
real, dimension(la,lw)             :: v,v0, gh
real, dimension(la,lk,lw)          :: vt
real, allocatable,dimension(:,:)   :: gw, gw1, mgama

! Variable for possible diverging iterations
real, dimension(la,lw)             :: oddgk, evengk, oddvw, evenvw
real, allocatable,dimension(:,:)   :: oddgw, evengw

real, dimension(autlk)             :: autk
real, dimension(la,autlk)          :: yaut, autkp, autu, autud, autup, vwp, tvwp, vwp1
real, dimension(la*(autlk-1),4)    :: autf

real, dimension(la*lw*lk)          :: den, den1

integer, parameter                 :: kxord=3
real, dimension(autlk+kxord)       :: xknot
real, dimension(la,autlk)          :: bscoef

real, dimension(lw+kxord)          :: wknot
real, dimension(lw)                :: vwcoef

real:: fv1,fv2

contains

function u(c)                     ! utility function
	real, intent(in) :: c
	real             :: u
	u   =  (c**(1.0-sigma)-1.0)/(1.0-sigma)
end function u

function cc(h)                    ! Inverse of utility function
	real, intent(in) :: h
	real             :: cc
	cc   =  (h*(1.0-sigma)+1.0)**(1.0/(1.0-sigma))
end function cc

function dcc(h)                   ! First derivative of inverse of U
	real, intent(in) :: h
	real             :: dcc
	dcc  =  (h*(1.0-sigma)+1.0)**(sigma/(1.0-sigma))
end function dcc

function ddcc(h)                  ! Second derivative of inverse of U
	real, intent(in) :: h
	real             :: ddcc
	ddcc  =  sigma * (h*(1.0-sigma) + 1.0)**((2.0*sigma-1.0)/(1.0-sigma))
end function ddcc

END MODULE penfvar
