subroutine Simu(simua, isimua, simuk0, simuw0)

!*******************************************************
! Simulation for the complete market:
! Input:
!       Productivity shock process A and P
!       Invariant distribution over states (A,K,B): miu (in CRS format)
!
!       miu(la*lk*lb):        stationary distribution
!
! Output:
!       Following simulation results for NC countries and T period
!       (1) Initial year state (A0, K0, B0)
!       (2) A(t) for t = 2: T-1
!
!       simua(NC,T):                simulated productivity process
!       simuk0(NC,1):               simulated capital stock
!       simub0(NC,1):               simulated bonds holding
!
! Parameters
!       T:                          number of years to be simulated
!       NC:                         number of countries to be simulated
!       la:                         number of productivity levels
!       lk:                         number of capital grid
!       lb:                         number of bond grid
!
!*******************************************************

USE simuvar

implicit none

! Results
integer, intent(out), dimension(T+1,NC)     :: isimua
real, intent(out), dimension(T+1,NC)        :: simua
real, intent(out), dimension(1,NC)          :: simuk0,simuw0

! Working arrays
integer                    :: i, j, m, s1

integer, dimension(NC)     :: state
real, dimension(NC)        :: tran1
real, dimension(T)         :: tran
real, dimension(T,NC)      :: ranp

real, dimension(la)        :: invp, cuminvp
complex, dimension(la)     :: eval
complex, dimension(la,la)  :: evec
real, dimension(la)        :: reval
real, dimension(la,la)     :: revec, tmpm, invevec, tmpm1, tmpmm

!***********************************************************************************
! Simulation
! Step (1)
! Generate T*nc random numbers
 
!s1=1          ! Select 9
!call rnset(s1)

!Draw random number for initial state  according to invariant distribution invp
call drnun(NC,tran1)
do i=1, NC
	do m = 1,la*lk*lw
		if (cummiu(m) >= tran1(i)) then
			ia           = ceiling(real(m)/real(lw*lk))
			iw           = ceiling(real(m-(ia-1)*lk*lw)/real(lk))
			ik           = m - (ia-1)*lw*lk - (iw-1)*lk
			isimua(1,i)  = ia
			simua(1,i)   = A(ia)  ! Initial productivity shock
			simuw0(1,i)  = W(iw)  ! Initital utility promises
			simuk0(1,i)  = K(ik)  ! Initial capital 
			exit
		end if
	end do
end do

!Random number for the following state a'
tran = 0.0
do i=1,nc
	call drnun(T,tran)
	ranp(:,i)=tran
end do

! Find the following states for each country
do i=1,nc              
	do j=2,T+1                      ! Iterate over periods 2 : T
		isimua(j,i)  = 1
		do m = 1,la
			if (cumpai(isimua(j-1,i),m)>=ranp(j-1,i)) then
				isimua(j,i) = m
				simua(j,i)  = a(m)
				exit
			end if
		end do
	end do                         ! end of simulation for T
end do                             ! end of simulation for nc 

END subroutine Simu
