!**************************************************************************
! Estimate Regime-Switching Process
!
! Method: maximum likelihood
!
!    y(t)=z(t)'beta(s(t))+u(st)
!                 u(st)~ i.i.d. N(0, sigma(st)^2)
!
!    y(t)       : series interested in here productivity shock
!    z(t)       : expaining variables including lagged values of y
!    s(t)       : an unobserved N-regimes with Markov chain
!
! Parameters:
!    Nc         : number of countries in the sample
!    T          : number of period        
!    Ne         : number of explaining variables
!    N          : number of regimes 
!
! Unknowns: 
!    rou        : starting probability
!    {p(ij)}    : transistion matrix(N*N)
!    beta(Ne*N) : persistence parameter and constant term in each regime
!    sigma(1*N) : the variance of shock in each regime
!
! Input:
!	 dAshock09.dat: generated from Matlab code computeA.m
!                 The detrended (using world common trend) productivity process
!
! Reference: Chapter 22, Time series analysis, James D. Hamilton
!            Jame D. Hamilton(1990), Journal of econometrcs 45 (39-70)
!            Hamilton (1996), Specification testing in Markov-switching time-series model
!                             Journal of Econometrica 70
!
! Author: Bai and Zhang
! March 20, 2009
!**************************************************************************
Program newregime

USE regimeVar

IMPLICIT NONE

! Working arrays
real*8                                  :: dip
real*8, dimension(N)                    :: sigma0,pai0,mu0,rou0
real*8                                  :: lamda0
real*8, dimension(N,N)                  :: p0

real*8, dimension(NC,T,N)              :: allsaiT

integer                                 :: i
real*8                                 :: setsai(T,N),setsai2(T,N,N),eta(NC,T,N),setfy(T) 


open(unit=5, file='./input/dAshock09.dat')
open(unit=9, file='./input/default09.dat')
open(unit=7, file='./input/guess3.dat')
open(unit=15, file='./output/newregime.out')
open(unit=8, file='./output/newregime.inp')

!****************************************
! load in TFP data and initial guess of parameters
call readin

!***************************************
! Reorganize data
y            = dpshock(:,2:T+1)
z            = dpshock(:,1:T)
z1          = 1.0
cdefault = cc(:,2:T+1)

dip = 2.0

do
	if (dip<=crit) exit
	call transpara(mu0,rou0,sigma0,pai0,p0,dip,lamda0,allsaiT)
	print *, dip
	mu = mu0
	rou = rou0
	sigma = sigma0
	pai = pai0
	p = p0
	lamda=0.0
end do

do i=1,NC
	call probf(i,setsai,setsai2,eta(i,:,:),setfy)
end do

call regimestd

call invariantp(N,p,pai)

call writeout
END PROGRAM newregime




