Module regimeVar


! Control variables
integer, parameter                   :: NC = 64, N = 3, Ne = 2, T =43, readp = 0,icalamda=0
real*8, parameter                    ::  crit = 1.0e-5, pi=3.14159265
real*8, dimension(N)                 :: phi

! Parameters to solve
real*8, dimension(N)                 :: rou, mu,ermu,errou 
real*8, dimension(Ne,N)              :: beta
real*8, dimension(N)                 :: sigma,pai,ersigma
real*8, dimension(N,N)               :: p,erp
real*8                               :: lamda

real*8,dimension(NC,T,N,3*N)         :: driv
real*8,dimension(N,N,N-1)            :: drivp

! Data
real*8, dimension(NC,T+1)            :: dpshock,cc

real*8, dimension(NC,T)              :: y,tz,yy,z,cdefault
real*8, dimension(T)                 :: z1

real*8                               :: covr(N*3+N*N)

Contains

!**********************************
subroutine probf(is,setsai,setsai2,eta,setfy)

integer,intent(in)          :: is
real*8,intent(out)            :: setsai(T,N),setsai2(T,N,N),eta(T,N),setfy(T) 

! Working arrays
integer                     :: m
real*8       	                :: sai(N),sai2(N,N),fy
real*8, dimension(T,N)        :: tempy,ttsigma
real*8, dimension(T,2)        :: tempz


sai   = pai ! sai(2|1)

tempy        = spread(y(is,:)-lamda*cdefault(is,:),2,N)
tempz(:,1)   = z1
tempz(:,2)   = z(is,:)
ttsigma      = spread(sigma,1,T)
	
! densities of each y|z,s, size: T*N
eta          = 1/(sqrt(2*pi)*ttsigma)*exp(-(tempy-matmul(tempz,beta))**2/(2*ttsigma**2))

! Construction of setsai, sai(st|Yt)
!                 setsai2, sai(st,st-1|Yt)
!                 setfy, p(yt|Yt-1)
do m=1,T
	fy             = sum(p*spread(eta(m,:),1,N)*spread(sai,2,N))
	sai2           = (p*spread(eta(m,:),1,N)*spread(sai,2,N))/fy
	sai            = sum(sai2,1)
	setsai(m,:)    = sai ! T*N
	setsai2(m,:,:) = sai2 
	setfy(m)       = fy
end do

End Subroutine probf

!**********************************
subroutine probb(is,setsaiT,setsaiT2,pai0)

integer,intent(in)                 :: is
real*8,intent(out)                   :: setsaiT(T,N),setsaiT2(T,N,N),pai0(N)

integer                            :: m,s,j,s1,s2
real*8                               :: eta(T,N),setsai(T,N),setsai2(T,N,N),setfy(T) 
real*8, dimension(N,N)               :: sai2, tsai2
real*8, dimension(N,N,N)             :: sai3, tsai3


call probf(is,setsai,setsai2,eta,setfy)

!--------------------------------
! Construct p(st,st-1|yT)
	
do m=1,T-1
	do s=1,N
		sai3(s,:,:) = p*spread(eta(m+1,:),1,N)*spread(setsai2(m,s,:),2,N)/setfy(m+1)
	end do
	do j=m+2,T
		do s1=1,N        ! st-1
			do s2=1,N    ! st
				tsai3(s1,s2,:)=sum(p*spread(eta(j,:),1,N)*spread(sai3(s1,s2,:),2,N),1)/setfy(j)
			end do
		end do
		sai3 = tsai3
	end do
	setsaiT2(m,:,:) = sum(sai3,3)
end do
setsaiT2(T,:,:) = setsai2(T,:,:)

!--------------------------------
! Construct p(st|yT) 
do m=1,T
	sai2 = setsai2(m,:,:)
	do j=m+1,T
		do s=1,N
			tsai2(s,:) = sum(p*spread(eta(j,:),1,N)*spread(sai2(s,:),2,N),1)/setfy(j)
		end do
		sai2 = tsai2
	end do
	
	if (m==1) then
		pai0 = sum(sai2,2)
	else
		setsaiT(m-1,:)=sum(sai2,2)
	end if
end do
setsaiT(T,:)=setsai(T,:)

end subroutine probb

!*****************************

Subroutine probs2(cal,phigh,plow,allsaiT,pai0)

real*8,intent(out), dimension(N,5)         :: cal
real*8,intent(out), dimension(N)           :: plow,pai0
real*8,intent(out), dimension(N,N)         :: phigh
real*8,intent(out), dimension(NC,T,N)      :: allsaiT


! Working arrays
real*8, dimension(T,N)                     :: tempy,tempzz

! Probabilities
real*8, dimension(T,N)                     :: setsaiT
real*8, dimension(T,N,N)                   :: setsaiT2

integer                                  :: i
real*8,dimension(N)                        :: A,B,C,D,F
real*8                                     :: tpai0(N)


A     = 0.0
B     = 0.0
C     = 0.0
D     = 0.0
F     = 0.0
phigh = 0.0
plow  = 0.0
pai0  = 0.0

beta(1,:) = mu*(1-rou)
beta(2,:) = rou

do i=1,NC	    

	call probb(i,setsaiT,setsaiT2,tpai0)
	allsaiT(i,:,:)=setsaiT
	pai0 = pai0 + tpai0
	
	tempzz = spread(z(i,:),2,N)
   	tempy  = spread(y(i,:)-lamda*cdefault(i,:),2,N)

	A = A + sum(tempy*setsaiT,1)
	B = B + sum(tempzz*setsaiT,1)
	C = C + sum(setsaiT,1)
	D = D + sum(tempy*tempzz*setsaiT,1)
	F = F + sum(tempzz**2*setsaiT,1)
	
	phigh = phigh + sum(setsaiT2(2:T,:,:),1)
	plow  = plow + sum(setsaiT(1:T-1,:),1)

	
end do

cal(:,1)=A
cal(:,2)=B
cal(:,3)=C
cal(:,4)=D
cal(:,5)=F


End subroutine probs2

!***********************************
Subroutine ydriv

integer                                  :: s,i,j
real*8                                     :: tt

!driv: mu1,mu2,mu3,rou1,rou2,rou3,sigma1,sigma2,sigma3,lamda

driv = 0.0
do i=1,NC
	do j=1,T
		do s=1,N
			tt = y(i,j)-lamda*cdefault(i,j)-(1-rou(s))*mu(s)-rou(s)*z(i,j)
			driv(i,j,s,s)= 1.0/sigma(s)**2*tt*(1-rou(s))  !dydmu
			driv(i,j,s,N+s)= 1.0/sigma(s)**2*tt*(z(i,j)-mu(s)) !dydrou
			driv(i,j,s,N*2+s)= -1.0/(2.0*sigma(s)**2)+1.0/(2.0*sigma(s)**4.0)*tt**2.0   !dydsigma
		end do
	end do
end do

End Subroutine ydriv

!***************************************
Subroutine pdriv

Implicit None

real*8,parameter                          :: eps=0.00001
integer                                 :: s1,s2
real*8                                    :: tp(N,N),td(N,N,N-1),invp(N),tinvp(N)


call invariantp(N,p,invp)

do s1=1,N        ! dp(s1=s)/dp(s1,s2)
	do s2=1,N-1
		tp = p
		if (tp(s1,N)<eps) then
			tp(s1,N)=p(s1,N)+eps
			tp(s1,s2)=p(s1,s2)-eps
			if (tp(s1,s2)<0.0) then
				print *, 'Error in calculating derivative to p !'
				exit
			end if
			call invariantp(N,tp,tinvp)
			td(:,s1,s2)=(invp-tinvp)/eps
			drivp(:,s1,s2) = td(:,s1,s2)/invp
		else
			tp(s1,N)=tp(s1,N)-eps
			tp(s1,s2)=p(s1,s2)+eps
			call invariantp(N,tp,tinvp)
			td(:,s1,s2)=(tinvp-invp)/eps
			drivp(:,s1,s2) = td(:,s1,s2)/invp
		end if

	end do
end do

End Subroutine pdriv


!***************************************
Subroutine sets3(pstd,it,eta,fy,pstd2,psai2,psai1)

Implicit None

integer,intent(in)   :: it
real*8,intent(in)      :: pstd(T+1,N,N,N),fy,eta(N)
real*8,intent(out)     :: pstd2(T+1,N,N,N),psai2(T+1,N,N),psai1(T+1,N)

real*8                 :: sai3s(N,N,N),tsai3(N,N,N)
integer              :: s,s1,s2,m

if (it>2) then
	sai3s=pstd(it-2,:,:,:)
else
	sai3s = pstd(1,:,:,:)
end if

do m=1,it-1
	do s=1,N            ! st
		do s1=1,N       ! stao-1
			do s2=1,N   ! stao
				tsai3(s1,s2,s) =sum(sai3s(s1,s2,:)*p(:,s)*eta(s))/fy
			end do
		end do
	end do

	sai3s = tsai3
	pstd2(it-1-m+1,:,:,:)=sai3s

	if (it>2) then
		psai2(it-1-m+1,:,:)=sum(sai3s,3)-sum(pstd(it-1-m+1,:,:,:),3)
		psai1(it-1-m+1,:)=sum(psai2(it-1-m+1,:,:),1)
	end if
end do

if (it==2) then
	psai2(1,:,:)=sum(sai3s,3)
	psai1(1,:)=sum(psai2(1,:,:),1)
end if

sai3s = 0.0
do s1=1,N
	sai3s(:,s1,s1)=sum(pstd2(it-1,:,:,s1),1)
end do

pstd2(it,:,:,:)=sai3s

	
End Subroutine sets3

END Module regimeVar





